==> opennms-33.1.5-core.tar.gz: opennms-33.1.5/opennms-dao-mock/src/main/java/org/opennms/netmgt/dao/mock/MockEventIpcManager.java <==
/*
 * Licensed to The OpenNMS Group, Inc (TOG) under one or more
 * contributor license agreements.  See the LICENSE.md file
 * distributed with this work for additional information
 * regarding copyright ownership.
 *
 * TOG licenses this file to You under the GNU Affero General
 * Public License Version 3 (the "License") or (at your option)
 * any later version.  You may not use this file except in
 * compliance with the License.  You may obtain a copy of the
 * License at:
 *
 *      https://www.gnu.org/licenses/agpl-3.0.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.  See the License for the specific
 * language governing permissions and limitations under the
 * License.
 */
package org.opennms.netmgt.dao.mock;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import javax.sql.DataSource;

import org.apache.commons.lang.builder.HashCodeBuilder;
import org.opennms.core.concurrent.LogPreservingThreadFactory;
import org.opennms.netmgt.config.api.EventConfDao;
import org.opennms.netmgt.config.api.EventdConfig;
import org.opennms.netmgt.dao.api.EventExpander;
import org.opennms.netmgt.events.api.EventForwarder;
import org.opennms.netmgt.events.api.EventIpcBroadcaster;
import org.opennms.netmgt.events.api.EventIpcManager;
import org.opennms.netmgt.events.api.EventListener;
import org.opennms.netmgt.events.api.EventProxy;
import org.opennms.netmgt.events.api.EventProxyException;
import org.opennms.netmgt.events.api.EventWriter;
import org.opennms.netmgt.events.api.model.IEvent;
import org.opennms.netmgt.events.api.model.ImmutableMapper;
import org.opennms.netmgt.xml.event.Event;
import org.opennms.netmgt.xml.event.Log;
import org.opennms.netmgt.xml.eventconf.Events;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;

public class MockEventIpcManager implements EventForwarder, EventProxy, EventIpcManager, EventIpcBroadcaster {
    private static final Logger LOG = LoggerFactory.getLogger(MockEventIpcManager.class);

    private static final AtomicInteger m_eventId = new AtomicInteger();

    static class ListenerKeeper {
    	final EventListener m_listener;
    	final Set<String> m_ueiList;

        ListenerKeeper(final EventListener listener, final Set<String> ueiList) {
            m_listener = listener;
            m_ueiList = ueiList;
        }

        @Override
        public int hashCode() {
            return new HashCodeBuilder(27, 31)
                .append(m_listener)
                .append(m_ueiList)
                .toHashCode();
        }

        @Override
        public boolean equals(final Object o) {
            if (o == null) return false;
            if (o instanceof ListenerKeeper) {
                final ListenerKeeper keeper = (ListenerKeeper) o;
                return m_listener.equals(keeper.m_listener) && (m_ueiList == null ? keeper.m_ueiList == null : m_ueiList.equals(keeper.m_ueiList));
            }
            return false;
        }

        private boolean eventMatches(final IEvent e) {
            if (m_ueiList == null)
                return true;
            return m_ueiList.contains(e.getUei());
        }

        public void sendEventIfAppropriate(final IEvent e) {
            if (eventMatches(e)) {
                m_listener.onEvent(e);
            }
        }
    }

    /**
     * This class implements {@link EventConfDao} but every call returns null.
     */
    public static class EmptyEventConfDao implements EventConfDao {
        @Override
        public void addEvent(final org.opennms.netmgt.xml.eventconf.Event event) {}

        @Override
        public void addEventToProgrammaticStore(final org.opennms.netmgt.xml.eventconf.Event event) {}

        @Override
        public org.opennms.netmgt.xml.eventconf.Event findByEvent(final Event matchingEvent) {
            return null;
        }

        @Override
        public org.opennms.netmgt.xml.eventconf.Event findByUei(final String uei) {
            return null;
        }

        @Override
        public String getEventLabel(final String uei) {
            return null;
        }

        @Override
        public Map<String, String> getEventLabels() {
            return null;
        }

        @Override
        public List<String> getEventUEIs() {
            return null;
        }

        @Override
        public List<org.opennms.netmgt.xml.eventconf.Event> getEvents(final String uei) {
            return null;
        }

        @Override
        public List<org.opennms.netmgt.xml.eventconf.Event> getEventsByLabel() {
            return null;
        }

        @Override
        public boolean isSecureTag(final String tag) {
            return false;
        }

        @Override
        public void reload() throws DataAccessException {}

        @Override
        public boolean removeEventFromProgrammaticStore(final org.opennms.netmgt.xml.eventconf.Event event) {
            return false;
        }

        @Override
        public void saveCurrent() {}

        @Override
        public Events getRootEvents() {
            return null;
        }
    }

    public static interface SendNowHook {
        public void beforeBroadcast(Event event);

        public void afterBroadcast(Event event);

        void finishProcessingEvents();
    }

    private EventAnticipator m_anticipator;
    
    private EventWriter m_eventWriter = new EventWriter() {
        @Override
        public void writeEvent(final Event e) {
            e.setDbid(m_eventId.incrementAndGet());
        }
    };

    private List<ListenerKeeper> m_listeners = new ArrayList<>();

    private AtomicInteger m_pendingEvents = new AtomicInteger();

    private volatile int m_eventDelay = 20;

    private boolean m_synchronous = true;
    
    private ScheduledExecutorService m_scheduler = null;

	private EventExpander m_expander = null;

	private SendNowHook m_sendNowHook;

	private int m_numSchedulerThreads = 1;

    public MockEventIpcManager() {
        m_anticipator = new EventAnticipator();
    }
    
    @Override
    public void addEventListener(final EventListener listener) {
        m_listeners.add(new ListenerKeeper(listener, null));
    }

    @Override
    public void addEventListener(final EventListener listener, final Collection<String> ueis) {
        m_listeners.add(new ListenerKeeper(listener, new HashSet<String>(ueis)));
    }

    @Override
    public void addEventListener(final EventListener listener, final String uei) {
        m_listeners.add(new ListenerKeeper(listener, Collections.singleton(uei)));
    }

    public int getEventListenerCount() {
        return m_listeners.size();
    }

    public void broadcastNow(final Event event) {
        broadcastNow(event, false);
    }

    @Override
    public void broadcastNow(final Event event, boolean synchronous) {
    	LOG.debug("Sending: {}", new EventWrapper(event));
        final List<ListenerKeeper> listeners = new ArrayList<ListenerKeeper>(m_listeners);
        final IEvent immutableEvent = ImmutableMapper.fromMutableEvent(event);
        for (final ListenerKeeper k : listeners) {
            k.sendEventIfAppropriate(immutableEvent);
        }
    }
    
    public void setEventWriter(final EventWriter eventWriter) {
        m_eventWriter = eventWriter;
    }

    public EventAnticipator getEventAnticipator() {
        return m_anticipator;
    }
    
    public void setEventExpander(final EventExpander expander) {
        m_expander  = expander;
    }

    public void setEventAnticipator(final EventAnticipator anticipator) {
        m_anticipator = anticipator;
    }

    @Override
    public void removeEventListener(final EventListener listener) {
        m_listeners.remove(new ListenerKeeper(listener, null));
    }

    @Override
    public void removeEventListener(final EventListener listener, final Collection<String> ueis) {
        m_listeners.remove(new ListenerKeeper(listener, new HashSet<String>(ueis)));
    }

    @Override
    public void removeEventListener(final EventListener listener, final String uei) {
        m_listeners.remove(new ListenerKeeper(listener, Collections.singleton(uei)));
    }
    
    public synchronized void setEventDelay(final int millis) {
        m_eventDelay  = millis;
    }

    /**
     * @param event
     */
    public void sendEventToListeners(final Event event) {
        m_eventWriter.writeEvent(event);
        broadcastNow(event, false);
    }

    public void setSynchronous(final boolean syncState) {
        m_synchronous = syncState;
    }
    
    public boolean isSynchronous() {
        return m_synchronous;
    }

    public void setSendNowHook(SendNowHook hook) {
        m_sendNowHook = hook;
    }

    public SendNowHook getSendNowHook() {
        return m_sendNowHook;
    }

    public void setNumSchedulerThreads(int numThreads) {
        m_numSchedulerThreads = numThreads;
    }

    public int getNumSchedulerTheads() {
        return m_numSchedulerThreads;
    }

    @Override
    public void sendNow(final Event event) {
        sendNow(event, isSynchronous());
    }

    public synchronized void sendNow(final Event event, boolean synchronous) {
        // Expand the event parms
        if (m_expander != null) {
            m_expander.expandEvent(event);
        }
        m_pendingEvents.incrementAndGet();
        LOG.debug("StartEvent processing ({} remaining)", m_pendingEvents);
        LOG.debug("Received: {}", new EventWrapper(event));
        m_anticipator.eventReceived(event);

        final Runnable r = new Runnable() {
            @Override
            public void run() {
                try {
                    // Allows external classes to alter the timing/order of events
                    if (m_sendNowHook != null) {
                        m_sendNowHook.beforeBroadcast(event);
                    }

                    m_eventWriter.writeEvent(event);
                    broadcastNow(event, synchronous);
                    m_anticipator.eventProcessed(event);
                } finally {
                    synchronized(MockEventIpcManager.this) {
                        m_pendingEvents.decrementAndGet();
                        LOG.debug("Finished processing event ({} remaining)", m_pendingEvents);
                        MockEventIpcManager.this.notifyAll();
                    }

                    // Allows external classes to alter the timing/order of events
                    if (m_sendNowHook != null) {
                        m_sendNowHook.afterBroadcast(event);
                    }
                }
            }
        };

        if (synchronous) {
            r.run();
        } else {
            getScheduler().schedule(r, m_eventDelay, TimeUnit.MILLISECONDS);
        }
    }

    ScheduledExecutorService getScheduler() {
        if (m_scheduler == null) {
            m_scheduler = Executors.newScheduledThreadPool(getNumSchedulerTheads(),
                new LogPreservingThreadFactory(getClass().getSimpleName(), getNumSchedulerTheads())
            );
        }
        return m_scheduler;
    }

    @Override
    public void sendNow(final Log eventLog) {
        for (final Event event : eventLog.getEvents().getEventCollection()) {
            sendNow(event);
        }
    }

    @Override
    public void sendNowSync(Event event) {
        sendNow(event, true);
    }

    @Override
    public void sendNowSync(Log eventLog) {
        for (final Event event : eventLog.getEvents().getEventCollection()) {
            sendNow(event, true);
        }
    }

    /**
     * 
     */
    public synchronized void finishProcessingEvents() {
        // Allow the hook to unblock any pending events
        if (m_sendNowHook != null) {
            m_sendNowHook.finishProcessingEvents();
        }

        while (m_pendingEvents.get() > 0) {
        	LOG.debug("Waiting for event processing: ({} remaining)", m_pendingEvents);
            try {
                wait();
            } catch (final InterruptedException e) {
                // Do nothing
            }
        }
    }

    public EventdConfig getEventdConfig() {
        // TODO Auto-generated method stub
        return null;
    }

    public void setEventdConfig(final EventdConfig eventdConfig) {
        // TODO Auto-generated method stub
    }

    public void setDataSource(final DataSource instance) {
        // TODO Auto-generated method stub
        
    }
    
    
    

    /**
     * Resets the event listeners and resets the event anticipator.
     */
    public void reset() {
        m_listeners = new ArrayList<>();
        m_anticipator.reset();
    }

    @Override
    public void send(final Event event) throws EventProxyException {
        sendNow(event);
    }

    @Override
    public void send(final Log eventLog) throws EventProxyException {
        sendNow(eventLog);
    }

    @Override
    public boolean hasEventListener(final String uei) {
        for (final ListenerKeeper keeper : this.m_listeners) {
            if (keeper.m_ueiList.contains(uei)) {
                return true;
            }
        }

        return false;
    }
}

==> opennms-35.0.0-core.tar.gz: opennms-35.0.0/opennms-dao-mock/src/main/java/org/opennms/netmgt/dao/mock/MockEventIpcManager.java <==
/*
 * Licensed to The OpenNMS Group, Inc (TOG) under one or more
 * contributor license agreements.  See the LICENSE.md file
 * distributed with this work for additional information
 * regarding copyright ownership.
 *
 * TOG licenses this file to You under the GNU Affero General
 * Public License Version 3 (the "License") or (at your option)
 * any later version.  You may not use this file except in
 * compliance with the License.  You may obtain a copy of the
 * License at:
 *
 *      https://www.gnu.org/licenses/agpl-3.0.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.  See the License for the specific
 * language governing permissions and limitations under the
 * License.
 */
package org.opennms.netmgt.dao.mock;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import javax.sql.DataSource;

import org.apache.commons.lang.builder.HashCodeBuilder;
import org.opennms.core.concurrent.LogPreservingThreadFactory;
import org.opennms.netmgt.config.api.EventConfDao;
import org.opennms.netmgt.config.api.EventdConfig;
import org.opennms.netmgt.dao.api.EventExpander;
import org.opennms.netmgt.events.api.EventForwarder;
import org.opennms.netmgt.events.api.EventIpcBroadcaster;
import org.opennms.netmgt.events.api.EventIpcManager;
import org.opennms.netmgt.events.api.EventListener;
import org.opennms.netmgt.events.api.EventProxy;
import org.opennms.netmgt.events.api.EventProxyException;
import org.opennms.netmgt.events.api.EventWriter;
import org.opennms.netmgt.events.api.model.IEvent;
import org.opennms.netmgt.events.api.model.ImmutableMapper;
import org.opennms.netmgt.model.EventConfEvent;
import org.opennms.netmgt.xml.event.Event;
import org.opennms.netmgt.xml.event.Log;
import org.opennms.netmgt.xml.eventconf.Events;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;

public class MockEventIpcManager implements EventForwarder, EventProxy, EventIpcManager, EventIpcBroadcaster {
    private static final Logger LOG = LoggerFactory.getLogger(MockEventIpcManager.class);

    private static final AtomicLong m_eventId = new AtomicLong();

    static class ListenerKeeper {
    	final EventListener m_listener;
    	final Set<String> m_ueiList;

        ListenerKeeper(final EventListener listener, final Set<String> ueiList) {
            m_listener = listener;
            m_ueiList = ueiList;
        }

        @Override
        public int hashCode() {
            return new HashCodeBuilder(27, 31)
                .append(m_listener)
                .append(m_ueiList)
                .toHashCode();
        }

        @Override
        public boolean equals(final Object o) {
            if (o == null) return false;
            if (o instanceof ListenerKeeper) {
                final ListenerKeeper keeper = (ListenerKeeper) o;
                return m_listener.equals(keeper.m_listener) && (m_ueiList == null ? keeper.m_ueiList == null : m_ueiList.equals(keeper.m_ueiList));
            }
            return false;
        }

        private boolean eventMatches(final IEvent e) {
            if (m_ueiList == null)
                return true;
            return m_ueiList.contains(e.getUei());
        }

        public void sendEventIfAppropriate(final IEvent e) {
            if (eventMatches(e)) {
                m_listener.onEvent(e);
            }
        }
    }

    /**
     * This class implements {@link EventConfDao} but every call returns null.
     */
    public static class EmptyEventConfDao implements EventConfDao {
        @Override
        public void addEvent(final org.opennms.netmgt.xml.eventconf.Event event) {}

        @Override
        public org.opennms.netmgt.xml.eventconf.Event findByEvent(final Event matchingEvent) {
            return null;
        }

        @Override
        public org.opennms.netmgt.xml.eventconf.Event findByUei(final String uei) {
            return null;
        }

        @Override
        public String getEventLabel(final String uei) {
            return null;
        }

        @Override
        public Map<String, String> getEventLabels() {
            return null;
        }

        @Override
        public List<String> getEventUEIs() {
            return null;
        }

        @Override
        public List<org.opennms.netmgt.xml.eventconf.Event> getEvents(final String uei) {
            return null;
        }

        @Override
        public List<org.opennms.netmgt.xml.eventconf.Event> getEventsByLabel() {
            return null;
        }

        @Override
        public boolean isSecureTag(final String tag) {
            return false;
        }

        @Override
        public void reload() throws DataAccessException {}

        @Override
        public Events getRootEvents() {
            return null;
        }

        @Override
        public void loadEventsFromDB(List<EventConfEvent> dbEvents) {

        }

    }

    public static interface SendNowHook {
        public void beforeBroadcast(Event event);

        public void afterBroadcast(Event event);

        void finishProcessingEvents();
    }

    private EventAnticipator m_anticipator;
    
    private EventWriter m_eventWriter = new EventWriter() {
        @Override
        public void writeEvent(final Event e) {
            e.setDbid(m_eventId.incrementAndGet());
        }
    };

    private List<ListenerKeeper> m_listeners = new ArrayList<>();

    private AtomicInteger m_pendingEvents = new AtomicInteger();

    private volatile int m_eventDelay = 20;

    private boolean m_synchronous = true;
    
    private ScheduledExecutorService m_scheduler = null;

	private EventExpander m_expander = null;

	private SendNowHook m_sendNowHook;

	private int m_numSchedulerThreads = 1;

    public MockEventIpcManager() {
        m_anticipator = new EventAnticipator();
    }
    
    @Override
    public void addEventListener(final EventListener listener) {
        m_listeners.add(new ListenerKeeper(listener, null));
    }

    @Override
    public void addEventListener(final EventListener listener, final Collection<String> ueis) {
        m_listeners.add(new ListenerKeeper(listener, new HashSet<String>(ueis)));
    }

    @Override
    public void addEventListener(final EventListener listener, final String uei) {
        m_listeners.add(new ListenerKeeper(listener, Collections.singleton(uei)));
    }

    public int getEventListenerCount() {
        return m_listeners.size();
    }

    public void broadcastNow(final Event event) {
        broadcastNow(event, false);
    }

    @Override
    public void broadcastNow(final Event event, boolean synchronous) {
    	LOG.debug("Sending: {}", new EventWrapper(event));
        final List<ListenerKeeper> listeners = new ArrayList<ListenerKeeper>(m_listeners);
        final IEvent immutableEvent = ImmutableMapper.fromMutableEvent(event);
        for (final ListenerKeeper k : listeners) {
            k.sendEventIfAppropriate(immutableEvent);
        }
    }
    
    public void setEventWriter(final EventWriter eventWriter) {
        m_eventWriter = eventWriter;
    }

    public EventAnticipator getEventAnticipator() {
        return m_anticipator;
    }
    
    public void setEventExpander(final EventExpander expander) {
        m_expander  = expander;
    }

    public void setEventAnticipator(final EventAnticipator anticipator) {
        m_anticipator = anticipator;
    }

    @Override
    public void removeEventListener(final EventListener listener) {
        m_listeners.remove(new ListenerKeeper(listener, null));
    }

    @Override
    public void removeEventListener(final EventListener listener, final Collection<String> ueis) {
        m_listeners.remove(new ListenerKeeper(listener, new HashSet<String>(ueis)));
    }

    @Override
    public void removeEventListener(final EventListener listener, final String uei) {
        m_listeners.remove(new ListenerKeeper(listener, Collections.singleton(uei)));
    }
    
    public synchronized void setEventDelay(final int millis) {
        m_eventDelay  = millis;
    }

    /**
     * @param event
     */
    public void sendEventToListeners(final Event event) {
        m_eventWriter.writeEvent(event);
        broadcastNow(event, false);
    }

    public void setSynchronous(final boolean syncState) {
        m_synchronous = syncState;
    }
    
    public boolean isSynchronous() {
        return m_synchronous;
    }

    public void setSendNowHook(SendNowHook hook) {
        m_sendNowHook = hook;
    }

    public SendNowHook getSendNowHook() {
        return m_sendNowHook;
    }

    public void setNumSchedulerThreads(int numThreads) {
        m_numSchedulerThreads = numThreads;
    }

    public int getNumSchedulerTheads() {
        return m_numSchedulerThreads;
    }

    @Override
    public void sendNow(final Event event) {
        sendNow(event, isSynchronous());
    }

    public synchronized void sendNow(final Event event, boolean synchronous) {
        // Expand the event parms
        if (m_expander != null) {
            m_expander.expandEvent(event);
        }
        m_pendingEvents.incrementAndGet();
        LOG.debug("StartEvent processing ({} remaining)", m_pendingEvents);
        LOG.debug("Received: {}", new EventWrapper(event));
        m_anticipator.eventReceived(event);

        final Runnable r = new Runnable() {
            @Override
            public void run() {
                try {
                    // Allows external classes to alter the timing/order of events
                    if (m_sendNowHook != null) {
                        m_sendNowHook.beforeBroadcast(event);
                    }

                    m_eventWriter.writeEvent(event);
                    broadcastNow(event, synchronous);
                    m_anticipator.eventProcessed(event);
                } finally {
                    synchronized(MockEventIpcManager.this) {
                        m_pendingEvents.decrementAndGet();
                        LOG.debug("Finished processing event ({} remaining)", m_pendingEvents);
                        MockEventIpcManager.this.notifyAll();
                    }

                    // Allows external classes to alter the timing/order of events
                    if (m_sendNowHook != null) {
                        m_sendNowHook.afterBroadcast(event);
                    }
                }
            }
        };

        if (synchronous) {
            r.run();
        } else {
            getScheduler().schedule(r, m_eventDelay, TimeUnit.MILLISECONDS);
        }
    }

    ScheduledExecutorService getScheduler() {
        if (m_scheduler == null) {
            m_scheduler = Executors.newScheduledThreadPool(getNumSchedulerTheads(),
                new LogPreservingThreadFactory(getClass().getSimpleName(), getNumSchedulerTheads())
            );
        }
        return m_scheduler;
    }

    @Override
    public void sendNow(final Log eventLog) {
        for (final Event event : eventLog.getEvents().getEventCollection()) {
            sendNow(event);
        }
    }

    @Override
    public void sendNowSync(Event event) {
        sendNow(event, true);
    }

    @Override
    public void sendNowSync(Log eventLog) {
        for (final Event event : eventLog.getEvents().getEventCollection()) {
            sendNow(event, true);
        }
    }

    /**
     * 
     */
    public synchronized void finishProcessingEvents() {
        // Allow the hook to unblock any pending events
        if (m_sendNowHook != null) {
            m_sendNowHook.finishProcessingEvents();
        }

        while (m_pendingEvents.get() > 0) {
        	LOG.debug("Waiting for event processing: ({} remaining)", m_pendingEvents);
            try {
                wait();
            } catch (final InterruptedException e) {
                // Do nothing
            }
        }
    }

    public EventdConfig getEventdConfig() {
        // TODO Auto-generated method stub
        return null;
    }

    public void setEventdConfig(final EventdConfig eventdConfig) {
        // TODO Auto-generated method stub
    }

    public void setDataSource(final DataSource instance) {
        // TODO Auto-generated method stub
        
    }
    
    
    

    /**
     * Resets the event listeners and resets the event anticipator.
     */
    public void reset() {
        m_listeners = new ArrayList<>();
        m_anticipator.reset();
    }

    @Override
    public void send(final Event event) throws EventProxyException {
        sendNow(event);
    }

    @Override
    public void send(final Log eventLog) throws EventProxyException {
        sendNow(eventLog);
    }

    @Override
    public boolean hasEventListener(final String uei) {
        for (final ListenerKeeper keeper : this.m_listeners) {
            if (keeper.m_ueiList.contains(uei)) {
                return true;
            }
        }

        return false;
    }

    public static void setEventIdStart(long l) {
        m_eventId.set(l);
    }
}
